/*
 * Created on 2004-3-2
 */
package org.cneclipse.multiport;

import java.io.*;
import java.net.*;
import java.util.*;
import org.gudy.azureus2.plugins.*;
import org.gudy.azureus2.plugins.download.*;
import org.gudy.azureus2.plugins.torrent.*;

/**
 * @author Leon
 */
public class MultiPortPlugin implements Plugin {

	private Map ports;
	private static final String CONFIG_URL = "http://bt.leonzhao.com/azureus/port_update.config";
	private long lastUpdate;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.gudy.azureus2.plugins.Plugin#initialize(org.gudy.azureus2.plugins.PluginInterface)
	 */
	public void initialize(PluginInterface pluginInterface) throws PluginException {
		final String configFile = pluginInterface.getPluginDirectoryName() + File.separator + "ports.config";
		final String updateFile = pluginInterface.getPluginDirectoryName() + File.separator + "update.config";
		updatePortConfig(configFile, updateFile);
		try {
			readConfig(configFile);
		} catch (Exception e) {
			e.printStackTrace();
			return;
		}
		final DownloadManager downloadManager = pluginInterface.getDownloadManager();
		final PluginConfig config = pluginInterface.getPluginconfig();
		downloadManager.addListener(new DownloadManagerListener() {

			public void downloadAdded(Download download) {
				try {
					Torrent pluginTorrent = download.getTorrent();
					TorrentAnnounceURLList announceURLList = pluginTorrent.getAnnounceURLList();
					TorrentAnnounceURLListSet[] sets = announceURLList.getSets();
					if (sets == null || sets.length < 2) {
						URL announceUrl = pluginTorrent.getAnnounceURL();
						int[] trackerPorts = (int[]) ports.get(announceUrl.getHost());
						if (trackerPorts != null && trackerPorts.length > 1) {
							for (int i = 0; i < trackerPorts.length; i++) {
								URL[] urlList = new URL[1];
								urlList[0] = new URL(announceUrl.getProtocol(), announceUrl.getHost(), trackerPorts[i],
										announceUrl.getFile());
								pluginTorrent.getAnnounceURLList().addSet(urlList);
							}
							pluginTorrent.save();
						}
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

			public void downloadRemoved(Download download) {
				// Do nothing
			}
		});
	}

	/**
	 * @param configFile
	 */
	private void updatePortConfig(final String configFile, final String updateFile) {
		try {
			lastUpdate = 0;
			try {
				BufferedReader in = new BufferedReader(new FileReader(updateFile));
				lastUpdate = Long.parseLong(in.readLine());
				in.close();
			} catch (Exception e) {
			}
			Thread t = new Thread() {

				/*
				 * @see java.lang.Thread#run()
				 */
				public void run() {
					try {
						URL url = new URL(CONFIG_URL);
						HttpURLConnection conn = (HttpURLConnection) url.openConnection();
						conn.connect();
						if (lastUpdate < conn.getLastModified()) {
							byte[] content = new byte[2048];
							ByteArrayOutputStream baos = new ByteArrayOutputStream();
							InputStream is = conn.getInputStream();
							int len;
							while((len = is.read(content)) > 0) {
								baos.write(content, 0, len);
							}
							is.close();
							FileOutputStream fos = new FileOutputStream(configFile);
							fos.write(baos.toByteArray());
							fos.flush();
							fos.close();
							fos = new FileOutputStream(updateFile);
							fos.write(("" + conn.getLastModified()).getBytes());
							fos.flush();
							fos.close();
							readConfig(configFile);
						}
						conn.disconnect();
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			};
			t.setDaemon(true);
			t.start();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param configFile
	 */
	private void readConfig(String configFile) throws Exception {
		ports = new Hashtable();
		String line, name = "", value = "";
		BufferedReader in = new BufferedReader(new FileReader(configFile));
		while ((line = in.readLine()) != null) {
			int n;
			name = value = "";
			line = line.trim();
			if (line.startsWith("#"))
				continue;
			n = line.indexOf("=");
			if (n == -1)
				continue;
			name = (line.substring(0, n)).trim();
			value = (line.substring(n + 1)).trim();
			if (value.equals(""))
				continue;
			int[] result = splitPorts(value);
			if (result != null)
				ports.put(name, result);
		}
		in.close();
	}

	/**
	 * @param message
	 * @return
	 */
	public static int[] splitPorts(String message) {
		if (message == null || message.length() == 0)
			return null;
		StringTokenizer st = new StringTokenizer(message, ",");
		Vector v = new Vector();
		while (st.hasMoreTokens()) {
			String word = st.nextToken();
			v.add(word);
		}
		int[] words = new int[v.size()];
		for (int i = 0; i < words.length; i++) {
			words[i] = Integer.parseInt((String) v.get(i));
		}
		return words;
	}
}
